<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag backup (low or high priority)
	 */

	load_f('mk_dir');

	class BACKUP	{

		var $DIR	=	FALSE;

		var $FORCE	=	FALSE;
		
		var $NEW_STRUCT = 	FALSE;
		
		var $DELAY	=	0;
		
		function __construct()	{

			global $__SQL;

			$this->SQL	=	&$__SQL;
			
			clearstatcache(TRUE);
			
			# Character set is available as of MySQL 5.1.38
			
			$this->set_character_set('5.1.38');
		}

		# check or try to create backup directory

		public function backup_directory($dir=FALSE)	{

			if (! $dir) return;

			# windows/unix

			$dir = safe_dir($dir);

			if (! file_exists($dir)) mk_dir($dir);
			
			if ((! is_dir($dir)) or (! is_writable($dir))) return 0;

			$this->DIR = $dir;
		}

		public function force()	{

			$this->FORCE = TRUE;
		}
		
		public function reset_structure()	{
		
			$this->NEW_STRUCT = TRUE;
		}
		
		public function low_priority($sec)	{
		
			$this->DELAY = $sec;
		}
		
		private function set_character_set($from_version)	{
		
			  $this->CHARACTER_SET = ' ';
			  
			  if (@version_compare($this->SQL->server_version(), $from_version, '>=')) $this->CHARACTER_SET = ' CHARACTER SET utf8 ';
		}
		
		private function dump_data($table)	{
		
			$file = "$this->BASE_DIR/$table.dat";
		
			# the select will not overwrite so delete files

			if (is_file($file)) @unlink($file);
			
			$file_esc = $this->SQL->mysqli->real_escape_string($file);
		
			$this->SQL->query("	SELECT * 
						INTO OUTFILE \"$file_esc\"
						$this->CHARACTER_SET
						FIELDS TERMINATED BY '\\t' ESCAPED BY '\\\'
						LINES TERMINATED BY '\\n'
						FROM $this->BASE_NAME.$table	");
		}
		
		private function dump_structure($table)	{
		
			$file = "$this->BASE_DIR/$table.str";
			
			# The structure is static, rewrite only if is forced
			
			if ((! $this->NEW_STRUCT) && is_file($file)) return;
		
			$this->SQL->query(" SHOW CREATE TABLE $this->BASE_NAME.$table ");

			if ($this->SQL->num_rows())	{

				$result	= $this->SQL->RESULT->fetch_assoc();

				$fh	= fopen($file, "wt");

				fwrite($fh, $result['Create Table']);

				fclose ($fh);
			}
		}
		
		# dump one base into separated directory

		private function basedump($BASE)	{
		
			$this->BASE_NAME = $BASE;
			
			$this->BASE_DIR  = $this->DIR.'/'.$this->BASE_NAME;

			$this->SQL->query("	SELECT SCHEMA_NAME 
						FROM INFORMATION_SCHEMA.SCHEMATA 
						WHERE SCHEMA_NAME = '$this->BASE_NAME'	");
			# base not exists

			if (! $this->SQL->num_rows()) return FALSE;

			# create folder to dump into

			if (! mk_dir($this->BASE_DIR)) return FALSE;

			# dump

			$this->SQL->query(" SHOW TABLES FROM $this->BASE_NAME ");

			# if nothing to dump this base is finished

			if (! $this->SQL->num_rows()) return TRUE;

			$this->SQL->IT1_RESULT = $this->SQL->RESULT;

			while (list($table) = $this->SQL->IT1_RESULT->fetch_row())	{
			
				$this->SQL->query(" LOCK TABLES $this->BASE_NAME.$table WRITE ");

				$this->dump_data($table);
				
				$this->dump_structure($table);

				$this->SQL->query('UNLOCK TABLES');
				
				if ($this->DELAY) sleep($this->DELAY);
			}

			return TRUE;
		}

		# note: integer return codes 	1 => previuous backup found or created new one 
		#				2 => no backup directory (depends that is error or not)
		#				3 => other process is in progress
		#				0 => backup error or maintenance in progress

		public function create()	{
		
			# check for maintenance
			
			if (is_file(BASEDIR.'tmp/maintenance.conf')) return 0;
		
			# backup is on daily base, verify that is 
			# not created yet if the the user not force 
			# to create it anyway (example before defragmentation)
			
			if (! $this->DIR) return 2;
			
			$today = date("m-d-y");
			
			$finished_mark = "$this->DIR/backup_finished.tmp";
			
			if (! $this->FORCE)	{
			
				  if (is_file($finished_mark) &&  (date("m-d-y", filemtime($finished_mark)) == $today)) return 1;
			}
			
			# check for running process
			
			$progress_mark = "$this->DIR/backup_started.tmp";
			
			if (is_file($progress_mark))	{
			
				  # previous backup process is finished ?
				  
				  $progress_mark_time = filemtime($progress_mark);
				  
				  # if mark is from today 
			
				  if (date("m-d-y", $progress_mark_time) == $today) return 3;
				  
				  # if mark is not from today, but 6 hours have not 
				  # been passed from previous (midnight prevent)
				  
				  if (($progress_mark_time + 21600) > time()) return 3;
				  
				  if (! @unlink($progress_mark)) return 0;
			}

			set_time_limit(0);
			
			if (is_file($finished_mark)) @unlink($finished_mark);
			
			@file_put_contents($progress_mark, $today);

			$this->SQL->query('	SELECT `year`
						FROM ebag.years
						WHERE `closed` IS NULL	');

			$this->OPENED	=	array();

			if ($this->SQL->num_rows())	{

				while ($RESULT = $this->SQL->RESULT->fetch_row()) $this->BASES[] = 'ebagdata'.$RESULT[0];
			}

			array_push($this->BASES, 'ebag', 'ebagstock');

			$code = 1;

			foreach ($this->BASES as $BASE)		{

				if (! $this->basedump($BASE))	{ $error=TRUE; break; }
			}

			if (isset($error)) $code = 0;

			@unlink($progress_mark);
			
			@file_put_contents($finished_mark, $today);

			return $code;
		}
	}

?>
