<?php	
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 *	eBag error handler
	 */
	 
	/**	
	 * 	usage :
	 * 	-	new Error_handler();	->	initialize the class
	 * 	-	start();		->	starts using error handler with defined settings
	 */

	class ERRHANDLER	{
	
		var $WARNINGSTEP	=	FALSE;
		var $NOTICESTEP		=	FALSE;
		var $CONF		=	FALSE;
		var $DEVELOPED		=	FALSE;

		function __construct()	{

			global $__CONF;

			$this->CONF = &$__CONF;
			
			foreach(array('REMOTE_ADDR', 'SERVER_NAME', 'SERVER_ADDR', 'HTTP_HOST') as $key) {

				  if (isset($_SERVER[$key])) {$this->{$key}=$_SERVER[$key];} else {$this->{$key}=FALSE;}
			}
		}
	
		/**
		 * Handlers main
		 */

		public function handler($errno, $errstr, $errfile, $errline, $errcontext)	{

			# user handled error with @

			if (! error_reporting()) return; 
			
			$errors	= array(	E_ERROR			=>	'Fatal Error',
						E_PARSE			=>	'Parsing Error',
						E_RECOVERABLE_ERROR	=>	'Catchable Fatal Error',
						E_CORE_ERROR		=>	'Core Error',
						E_COMPILE_ERROR		=>	'Compile Error',
						E_USER_ERROR		=>	'User Error'	);
						
			$warnings = array(	E_WARNING		=>	'Warning',
						E_CORE_WARNING		=>	'Core Warning',
						E_COMPILE_WARNING	=>	'Compile Warning',
						E_USER_WARNING		=>	'User Warning'	);
						
			$notices = array(	E_NOTICE		=>	'Notice',
						E_USER_NOTICE		=>	'User Notice',
						E_STRICT		=>	'Runtime Notice',
						E_DEPRECATED		=>	'Deprecated Notice',
						E_USER_DEPRECATED	=>	'User Deprecated Notice'	);
						
			# will be ignored ?
			
			if (($notice = isset($notices[$errno])) && $this->NOTICESTEP) return TRUE;
			
			if (isset($warnings[$errno]) && $this->WARNINGSTEP) return TRUE;

			# unlock tables

			global $__SQL;

			if (isset($__SQL))	{

				$__SQL->rollback();
				$__SQL->query('UNLOCK TABLES');
			}

			$error = 'Unknown Error';

			foreach(array('errors', 'warnings', 'notices') as $TYPE)	{
			
				if (isset(${$TYPE}[$errno])) {$error = ${$TYPE}[$errno]; break;}
			}
			
			# log
			
			if (ini_get('log_errors')) error_log("PHP $error:  $errstr in $errfile on line $errline.");
			
			$message = "<pre><font color=\"red\">\n$error: $errstr in $errfile in line $errline\n</font></pre>";

			# error message

			if (! $notice)	{

				ob_start();
				
				foreach($errcontext as $key=>$val) {if ((! in_array($key, array('__LANG', '__HANDLER', '__BOX'))) && $val) echo '$'.$key.'=>'.print_r($val, TRUE);}

				$var_trace = "<pre><font color=\"green\">Var trace:\n\n".ob_get_contents()."\n</font></pre>";

				ob_end_clean();
			}
			
			if (ini_get('display_errors')) {echo $message; if ($this->DEVELOPED && (! $notice)) echo $var_trace;}

			# send mail

			if ((! $notice) && (! $this->DEVELOPED) && $this->CONF->get('error','report_to'))	{

				if (! $host = $this->HTTP_HOST)	{

					if (! $host = $this->SERVER_NAME) $host = 'Ip: '.$this->SERVER_ADDR;
				}

				require_once(BASEDIR.'classes/smtpmail.class.php');

				$mail = new SmtpMail();

				$mail->host($this->CONF->get('mail', 'host'));
				$mail->port($this->CONF->get('mail', 'port'));
				$mail->secure($this->CONF->get('mail', 'secure'));
				$mail->user($this->CONF->get('mail', 'user'));
				$mail->password($this->CONF->get('mail', 'password'));
				$mail->to($this->CONF->get('error','report_to'));
				$mail->subject($host.' error log');
				$mail->message(wordwrap("<pre><font color=\"blue\">Error reporting from $host &nbsp;&nbsp;&nbsp;".@date("Y.m.d. h:m:s")."</font></pre>\n".$message.$var_trace, 70, "\n", true));

				if ($mail->send()) echo '<pre><font color="green">Report has been send to eBag administrator.</font></pre>';
			}

			exit('<pre><font color="red">Script execution halted.</font></pre>');
		}

		/**
		 * Sets the handler
		 */

		function start()	{

			set_error_handler(array(&$this, 'handler'));
		}
	}

?>
