<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 *	Check smtp server configuration
	 */

	class SMTPCheck	{
	
		var $EOL 		=	"\r\n";
		
		var $host		=	'';
		var $user		=	'';
		var $password		=	'';
		var $secure		=	'';
		var $port		=	'';
		
		var $SERVER_NAME	=	'localhost';
		
		function __construct()	{
		
			if (isset($_SERVER['SERVER_NAME'])) $this->SERVER_NAME = $_SERVER['SERVER_NAME'];
		}
	
		public function host($host)	{
		
			$this->host = $host;
		}
		
		public function user($user)	{
		
			$this->user = $user;
		}
		
		public function password($password)	{
		
			$this->password = $password;
		}

		public function secure($secure)	{
		
			$this->secure = $secure;
		}
		
		public function port($port)	{
		
			$this->port = $port;
		}

		private function command_send($command, $expected_code)	{

			if (! @fputs($this->smtp, $command)) return FALSE;

			$response=$this->get_response();

			$response_ok=FALSE;

			if ($response)	{

				if ((strpos($command, 'EHLO') !== FALSE) or (strpos($command, 'HELO') !== FALSE))	{

					if (strpos(strtoupper($response), 'AUTH') !== FALSE) $this->server_auth=TRUE;
				}

				if (substr($response, 0, 3) == $expected_code)	$response_ok=TRUE;
			}

			return $response_ok;
		}

		private function get_response()		{

			if (! $response = @fread($this->smtp, 1)) return FALSE;

			if (! $state = socket_get_status($this->smtp)) return FALSE;

			if ($state['timed_out']) return FALSE;

			if (! $left=@fread($this->smtp, $state['unread_bytes'])) return FALSE;

			return $response.=$left;
		}

		private function disconnect($err=FALSE)	{

			if ($err)	{
				@fputs($this->smtp, 'RSET'.$this->EOL);
				@fputs($this->smtp, 'QUIT'.$this->EOL);
			}

			@fclose($this->smtp);
		}
		
		public function check()	{
		
			# for any case
			
			set_time_limit(0);
		
			if ($this->secure) $this->secure = $this->secure.'://';

			$STARTTLS = FALSE;
			
			$this->smtp = @fsockopen($this->secure.$this->host, $this->port, $errno, $errstr, 30);
				
			if ((! $this->smtp) && (strpos(strtolower($this->secure), 'tls') !== FALSE))	{

				# first check if tls connection is then try to initialize with STARTTLS
					
				if ($this->smtp = @fsockopen($this->host, $this->port, $errno, $errstr, 30)) $STARTTLS = TRUE;
			}

			# empty the first response

			$this->get_response();

			if (! $this->command_send('EHLO '.$this->SERVER_NAME.$this->EOL, 250))	{

				if (! $this->command_send('HELO '.$this->SERVER_NAME.$this->EOL, 250)) {

					$this->disconnect(TRUE);

					return FALSE;
				}
			}

			# initialize tls connection by need
			
			if ($STARTTLS)	{
			
				if (!  $this->command_send('STARTTLS'.$this->EOL, 220)) {
				
					$this->disconnect();

					return FALSE;
					
				}	else	  {
				
					stream_socket_enable_crypto($this->smtp, true, STREAM_CRYPTO_METHOD_TLS_CLIENT);
				}
			}
			
			if (	(isset($this->server_auth)) && 
				((is_empty($this->user)) or (is_empty($this->password)))	)	{

					$this->disconnect();

					return FALSE;
			}

			if (! $this->command_send("AUTH LOGIN".$this->EOL, 334))	{

				$this->disconnect();

				return FALSE;
			}

			if (! $this->command_send(base64_encode($this->user).$this->EOL, 334))	{

				$this->disconnect();

				return FALSE;
			}

			if (! $this->command_send(base64_encode($this->password).$this->EOL, 235))	{

				$this->disconnect();

				return FALSE;
			}

			$this->disconnect();
			
			return TRUE;
		}
	}


?>