<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag save changed configuration
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('is_int_positive');
	load_f('is_real_positive');

	class CONFIGURATIONSAVE	{

		function __construct()	{

			global $__SQL;

			$this->SQL = &$__SQL;

			$this->STOCKBASE = 'ebagstock';
		}

		private function lock()	 {

			$this->SQL->query("	LOCK TABLES 	$this->STOCKBASE.services WRITE, 
								$this->STOCKBASE.finished_goods_stock WRITE	");
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}
		
		private function is_writable_backup_dir($dir)	{
		
			$dir = safe_dir($dir);
		
			# www
	
			if (is_empty($dir) or (! is_dir($dir)) or (! is_writable($dir))) return FALSE;
			
			# mysql
			
			$file = "$dir/is_writable.tmp";
			
			$file_esc = $this->SQL->mysqli->real_escape_string($file);
				  
			@unlink($file);
				  
			if (! $this->SQL->mysqli->query(" SELECT 'is_writable' INTO OUTFILE \"$file_esc\" ")) return FALSE;
				  
			@unlink($file);
			
			return TRUE;
		}

		public function save($DATA, $CONFS)	{

			set_time_limit(0);

			# delete % if is there in tax

			$DATA['default_tax_for_service'] = str_replace('%', '', $DATA['default_tax_for_service']);
			
			$DATA['default_tax_for_sales'] = str_replace('%', '', $DATA['default_tax_for_sales']);

			# check user input

			if (	(is_empty($DATA['currency'])) or
				(is_empty($DATA['valuta'])) or
				(! is_real_positive($DATA['exchange_rate'])) or
				(! is_real_positive($DATA['default_tax_for_service'])) or
				(! is_real_positive($DATA['default_tax_for_sales'])) or
				(! is_int_positive($DATA['default_deadline_for_service'])) or
				(! is_int_positive($DATA['default_deadline_for_shipment'])) or
				(! is_int_positive($DATA['increment'])) or
				(! is_int_positive($DATA['dayscheck']))	) return 'usererror'; 

			$DATA['default_tax_for_service'] .= '%';
			
			$DATA['default_tax_for_sales'] .= '%';

			# verify the SMTP settings if is defined
			
			if (	(! is_empty($DATA['host']))	or
				(! is_empty($DATA['user']))	or
				(! is_empty($DATA['port']))	)	{
				
				if ($DATA['secure'] == 'none') $DATA['secure'] = '';

				$smtpcheckpassed = FALSE;
				
				if ($DATA['password'] == $DATA['re_password'])	{
			
					  load_c('SMTPCheck');
					  
					  $smtp = new SMTPCheck();
					  
					  $smtp->host($DATA['host']);
					  $smtp->user($DATA['user']);
					  $smtp->password($DATA['password']);
					  $smtp->secure($DATA['secure']);
					  $smtp->port($DATA['port']);
					  
					  if ($smtp->check()) $smtpcheckpassed = TRUE;
					  
					  unset($smtp);
				}
				
				if (! $smtpcheckpassed) return 'smtpsettingserror';
				
			}	else	 {
			
				$DATA['host'] = $DATA['user'] = $DATA['password'] = $DATA['secure'] = $DATA['port'] = '';
			}
			
			unset($DATA['re_password']);

			# check the mail syntax if is defined

			if (! is_empty($DATA['report_to']))	{

				if (function_exists('filter_var'))	{

					$valid=filter_var($DATA['report_to'], FILTER_VALIDATE_EMAIL);

				}	else	{

					$valid=@preg_match("/^[^@]*@[^@]*\.[^@]*$/", $DATA['report_to']);
				}

				if (! $valid) return 'mailsyntaxincorrect';
			}

			# verify the backup directory if is defined or automatic backup is set

			if ( ($DATA['directory'] or $DATA['daily_backup']) && (! $this->is_writable_backup_dir($DATA['directory'])) ) return 'backupdirerr';
				
			# verify clamav if is defined
				
			if (($DATA['clamavpath']) && (! is_readable($DATA['clamavpath']))) return 'clamav_not_found';

			# change configuration

			if (! is_writable(BASEDIR.'conf/eBag.ini')) return 'cannotsaveini';

			$configuration	=	new INICHANGE();

			$configuration	->	open(BASEDIR.'conf/eBag.ini');

			foreach ($CONFS as $key=>$section) $configuration->change($section, $key, $DATA[$key]);

			# save 

			$configuration	->	save();

			return TRUE;
		}
	}

?>
