<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag payment_forms (new, change, delete and set default)
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('is_int_positive');
	
	load_f('is_longer');

	class PAYMENTFORMS	{

		function __construct()	{

			global $__SQL;

			$this->SQL	=	&$__SQL;

			$this->readyears();			
		}

		private function lock()	{

			$TOLOCK	= '';

			foreach ($this->YEARS as $YEAR)	$TOLOCK .= ", ebagdata$YEAR.services_header WRITE, ebagdata$YEAR.offers_header WRITE, ebagdata$YEAR.orders_header WRITE ";

			$this->SQL->query('LOCK TABLES ebag.years WRITE, ebag.payments WRITE'.$TOLOCK);
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		private function readyears()	{

			$this->YEARS = array();
			
			$this->SQL->query("	SELECT `year` 
						FROM ebag.years 
						ORDER BY `year`	");

			while ($RESULT = $this->SQL->RESULT->fetch_row()) $this->YEARS[] = $RESULT[0];

		}

		private function is_in_use($paymentID)	{

			$FOUND = FALSE;

			foreach ($this->YEARS as $YEAR)	{

				$this->SQL->query("	(
								SELECT `personID` 
								FROM ebagdata$YEAR.offers_header 
								WHERE `payment_form`='$paymentID' AND `deleted` IS NOT NULL LIMIT 1
							)
							UNION ALL
							(
								SELECT `personID` 
								FROM ebagdata$YEAR.orders_header 
								WHERE `payment_form`='$paymentID' AND `deleted` IS NOT NULL LIMIT 1
							)	
							UNION ALL
							(
								SELECT `personID` 
								FROM ebagdata$YEAR.services_header 
								WHERE `payment_form`='$paymentID' AND `deleted` IS NOT NULL LIMIT 1
							)	");

				if ($this->SQL->num_rows())	{

					$FOUND = TRUE;

					break;
				}
			}

			return $FOUND;
		}

		public function defaultpayment($paymentID)	{

			$this->SQL->transaction();

			# note: no need to lock all tables
		
			$this->SQL->query('LOCK TABLES ebag.payments WRITE');

			$this->SQL->query("	UPDATE ebag.payments
						SET `default`=NULL 
						WHERE `default` IS NOT NULL	");

			$this->SQL->query("	UPDATE ebag.payments
						SET `default`='1' 
						WHERE `personID`='$paymentID'	LIMIT 1");

			$this->SQL->commit();

			$this->unlock();

			return TRUE;		
		}

		public function change($paymentID, $NAME, $DAYS)	{

			if (	((is_empty($NAME)) && (is_empty($DAYS))) or 
				((! is_empty($DAYS)) && (! is_int_positive($DAYS)) && ($DAYS != 0)) or
				((! is_empty($NAME)) && (is_longer($NAME, 40)))	)	return 'usererror';

			$this->lock();

			if ($this->is_in_use($paymentID))	{

				$this->unlock();

				return 'cannotchangeusedpayment';
			}
		
			if (! is_empty($NAME))	{

				$NAME	=	$this->SQL->n_escape($NAME);

				# check the new name not exists once

				$this->SQL->query("	SELECT `personID`
							FROM ebag.payments
							WHERE `description`=$NAME LIMIT 1	");

				if ($this->SQL->num_rows())	{

					$this->unlock();

					return 'paymentformexists';
				}

				$this->SQL->query("	UPDATE ebag.payments
							SET `description`=$NAME
							WHERE `personID`='$paymentID' LIMIT 1	");
			}

			if (! is_empty($DAYS))	{

				$this->SQL->query("	UPDATE ebag.payments
							SET `expected_after_day`='$DAYS'
							WHERE `personID`='$paymentID' LIMIT 1	");
			}

			$this->unlock();
	
			return TRUE;

		}

		public function delete($paymentID)	{

			$this->lock();

			if ($this->is_in_use($paymentID))	{

				$this->unlock();

				return 'cannotdeleteusedpayment';
			}

			# check that is not default

			$this->SQL->query("	SELECT `personID` 
						FROM ebag.payments
						WHERE `personID`='$paymentID' AND `default` IS NOT NULL LIMIT 1	");

			if ($this->SQL->num_rows())	{

				$this->unlock();

				return 'cannotdeletedefaultpayment';
			}

			# check that is not the last

			$this->SQL->query("	SELECT `description`
						FROM ebag.payments	");

			if ($this->SQL->num_rows()==1)	{

				$this->unlock();

				return 'cannotdeletelastpayment';
			}

			$this->SQL->query("	DELETE 
						FROM ebag.payments
						WHERE `personID`='$paymentID' LIMIT 1	");

			$this->unlock();

			return TRUE;
		}

		public function addnew($NAME, $DAYS)	{

			if (	((is_empty($NAME)) or (is_longer($NAME, 40)) or (is_empty($DAYS))) or 
				((! is_int_positive($DAYS)) && ($DAYS != 0))	) return 'usererror';

			# note: no need to lock all tables

			$this->SQL->query('LOCK TABLES ebag.payments WRITE');

			$NAME	=	$this->SQL->n_escape($NAME);

			# check the new name not exists once

			$this->SQL->query("	SELECT `personID`
						FROM ebag.payments
						WHERE `description`=$NAME LIMIT 1	");

			if ($this->SQL->num_rows())	{

				$this->unlock();

				return 'paymentformexists';
			}

			$this->SQL->query("	INSERT INTO ebag.payments
						VALUES
						(NULL, $NAME, '$DAYS', NULL)	");

			$this->unlock();
	
			return TRUE;
		}
	}
?>
